package com.scpark.blecmd;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.IBinder;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView.ScaleType;
import android.widget.TextView;

import com.scpark.blecmd.service.BluetoothLeService;
import com.scpark.blecmd.util.LogUtil;
import com.umeng.analytics.MobclickAgent;

public class Bluemd extends Activity {

	private BluetoothLeService mBluetoothLeService;

	private final ServiceConnection mServiceConnection = new ServiceConnection() {
		@Override
		public void onServiceConnected(ComponentName componentName,
				IBinder service) {
			mBluetoothLeService = ((BluetoothLeService.LocalBinder) service)
					.getService();
			if (!mBluetoothLeService.initialize()) {
				LogUtil.e("Unable to initialize Bluetooth");
				finish();
			}
			LogUtil.e("mBluetoothLeService is okay");
		}

		@Override
		public void onServiceDisconnected(ComponentName componentName) {
			mBluetoothLeService = null;
		}
	};

	private final BroadcastReceiver mGattUpdateReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			final String action = intent.getAction();
			if (BluetoothLeService.ACTION_GATT_CONNECTED.equals(action)) {
				mTitle.setText(R.string.title_connecting);
				isConnected = true;
			} else if (BluetoothLeService.ACTION_GATT_DISCONNECTED
					.equals(action)) {
				isConnected = false;
				mTitle.setText(R.string.title_not_connected);
			} else if (BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED
					.equals(action)) {
				isConnected = true;
				firstTimeThread.restart();
				mTitle.setText(R.string.title_connected_to);
				mTitle.append(mConnectedDeviceName);
				alreadyUpdated = false;
				progressDialog = ProgressDialog.show(Bluemd.this, "请稍候...",
						"正在更新状态，请稍候...", true, true);
			} else if (BluetoothLeService.ACTION_DATA_AVAILABLE.equals(action)) {
				if (progressDialog != null) {
					progressDialog.dismiss();
				}
				byte[] datas = intent
						.getByteArrayExtra(BluetoothLeService.EXTRA_DATA);
				if (datas == null || datas.length != 2) {
					return;
				}
				int status = datas[0] & 0xff;
				int lightValue = 255 - datas[1] & 0xff;
				if (!alreadyUpdated) {
					if (status == 1) {
						onOff.setImageDrawable(getApplication().getResources()
								.getDrawable(R.drawable.button_on_metal));
						isON = true;
						onOff.setScaleType(ScaleType.FIT_XY);
					} else {
						onOff.setImageDrawable(getApplication().getResources()
								.getDrawable(R.drawable.button_off_metal));
						onOff.setScaleType(ScaleType.FIT_XY);
						isON = false;
					}
					lastProgress = lightValue;
					textView.setText("" + lastProgress);
					roundView.setProgress(lastProgress);
					alreadyUpdated = true;
				}
			}
		}
	};

	public String convertStringToHex(String str) {
		char[] chars = str.toCharArray();
		StringBuffer hex = new StringBuffer();
		for (int i = 0; i < chars.length; i++) {
			hex.append(Integer.toHexString((int) chars[i]));
		}
		return hex.toString();
	}

	private TextView mTitle;

	private int lastProgress = 0;
	private RoundView roundView;
	private TextView textView;
	private ImageButton buttonLeft;
	private ImageButton buttonRight;
	private ImageButton onOff;
	private ProgressDialog progressDialog;
	private FirstTimeThread firstTimeThread;
	private boolean isConnected = false;
	private boolean alreadyUpdated = false;
	public static final String PREFS_NAME = "MyPrefsFile";

	private int lastSendProgress = 0;
	private SharedPreferences preferences;
	private boolean isON = false;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		mTitle = (TextView) findViewById(R.id.title_left_text);
		mTitle.setText("LazyBone");
		mTitle = (TextView) findViewById(R.id.title_right_text);
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		if (mBluetoothAdapter == null) {
			finishDialogNoBluetooth();
			return;
		}

		preferences = getSharedPreferences(PREFS_NAME, 0);

		roundView = (RoundView) findViewById(R.id.roundview);
		textView = (TextView) findViewById(R.id.progressTextView);
		buttonLeft = (ImageButton) findViewById(R.id.buttonLeft);
		onOff = (ImageButton) findViewById(R.id.onoff);

		onOff.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				if (isON) {
					liveData("6f000d");
				} else {
					liveData("65000d");
				}
				isON = !isON;
				if (isON) {
					onOff.setImageDrawable(getApplication().getResources()
							.getDrawable(R.drawable.button_on_metal));
				} else {
					onOff.setImageDrawable(getApplication().getResources()
							.getDrawable(R.drawable.button_off_metal));
				}
				onOff.setScaleType(ScaleType.FIT_XY);
			}
		});
		buttonLeft.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				if (isConnected) {
					lastProgress--;
					if (lastProgress < 0)
						lastProgress = 0;
					lastSendProgress = lastProgress;
					textView.setText(String.valueOf(lastProgress));
					roundView.setProgress(lastProgress);
					if (Integer.toHexString(255 - lastProgress).length() == 1) {
						sendData("670"
								+ Integer.toHexString(255 - lastProgress)
								+ "0d");
					} else {
						sendData("67" + Integer.toHexString(255 - lastProgress)
								+ "0d");

					}
				}
			}
		});
		buttonRight = (ImageButton) findViewById(R.id.buttonRight);
		buttonRight.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				if (isConnected) {
					lastProgress++;
					if (lastProgress > 255)
						lastProgress = 255;
					lastSendProgress = lastProgress;

					textView.setText(String.valueOf(lastProgress));
					roundView.setProgress(lastProgress);
					if (Integer.toHexString(255 - lastProgress).length() == 1) {
						sendData("670"
								+ Integer.toHexString(255 - lastProgress)
								+ "0d");
					} else {
						sendData("67" + Integer.toHexString(255 - lastProgress)
								+ "0d");

					}
				}
			}
		});
		roundView.setRoundViewListener(new RoundViewListener() {
			@Override
			public void onMoved(int progress) {
				// TODO Auto-generated method stub
				LogUtil.i("Progress:" + progress);
				if (isConnected) {
					if (progress % 8 == 0
							|| (progress / 8 != lastSendProgress / 8)) {
						lastProgress = progress / 8 * 8;
						lastSendProgress = lastProgress;
						if (lastProgress > 255)
							lastProgress = 255;
						else if (lastProgress < 0)
							lastProgress = 0;
						textView.setText(String.valueOf(lastProgress));
						if (Integer.toHexString(256 - lastProgress).length() == 1) {
							sendData("670"
									+ Integer.toHexString(256 - lastProgress)
									+ "0d");
						} else {
							if (lastProgress == 0) {
								sendData("67"
										+ Integer
												.toHexString(255 - lastProgress)
										+ "0d");

							} else {
								sendData("67"
										+ Integer
												.toHexString(256 - lastProgress)
										+ "0d");
							}
						}
					} else {
						lastProgress = progress;
						textView.setText(String.valueOf(lastProgress));
						if (lastProgress == 255) {
							lastProgress = 256;
							sendData("670"
									+ Integer.toHexString(256 - lastProgress)
									+ "0d");
						}
					}
				} else {
					roundView.setProgress(progress);
				}
			}
		});
		Intent gattServiceIntent = new Intent(this, BluetoothLeService.class);
		LogUtil.e("Try to bindService="
				+ bindService(gattServiceIntent, mServiceConnection,
						BIND_AUTO_CREATE));
		registerReceiver(mGattUpdateReceiver, makeGattUpdateIntentFilter());
		roundView.setProgress(0);
		firstTimeThread = new FirstTimeThread();
		firstTimeThread.start();
	}

	private static IntentFilter makeGattUpdateIntentFilter() {
		final IntentFilter intentFilter = new IntentFilter();
		intentFilter.addAction(BluetoothLeService.ACTION_GATT_CONNECTED);
		intentFilter.addAction(BluetoothLeService.ACTION_GATT_DISCONNECTED);
		intentFilter
				.addAction(BluetoothLeService.ACTION_GATT_SERVICES_DISCOVERED);
		intentFilter.addAction(BluetoothLeService.ACTION_DATA_AVAILABLE);
		intentFilter.addAction(BluetoothDevice.ACTION_UUID);
		return intentFilter;
	}

	private class FirstTimeThread extends Thread {
		boolean flag = true;

		@Override
		public void run() {
			while (flag) {
				if (isConnected) {
					if (mBluetoothLeService != null) {
						byte[] datas = { 0x5B, 0x00, 0x0D };
						mBluetoothLeService.WriteBytes(datas);
					}
				}
				try {
					sleep(500);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
				}
			}
		}

		public void Stop() {
			flag = false;
		}

		public void restart() {
			flag = true;
		}
	}

	@Override
	public void onStart() {
		super.onStart();
		mEnablingBT = false;
	}

	private BluetoothAdapter mBluetoothAdapter = null;
	private boolean mEnablingBT = false;

	private static final int REQUEST_ENABLE_BT = 2;

	@Override
	protected void onPause() {
		// TODO Auto-generated method stub
		super.onPause();
		MobclickAgent.onPause(this);
	}

	@Override
	public synchronized void onResume() {
		super.onResume();
		MobclickAgent.onResume(this);
		if (!mEnablingBT) {
			if ((mBluetoothAdapter != null) && (!mBluetoothAdapter.isEnabled())) {
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setMessage(R.string.alert_dialog_turn_on_bt)
						.setIcon(android.R.drawable.ic_dialog_alert)
						.setTitle(R.string.alert_dialog_warning_title)
						.setCancelable(false)
						.setPositiveButton(R.string.alert_dialog_yes,
								new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog,
											int id) {
										mEnablingBT = true;
										Intent enableIntent = new Intent(
												BluetoothAdapter.ACTION_REQUEST_ENABLE);
										startActivityForResult(enableIntent,
												REQUEST_ENABLE_BT);
									}
								})
						.setNegativeButton(R.string.alert_dialog_no,
								new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog,
											int id) {
										finishDialogNoBluetooth();
									}
								});
				AlertDialog alert = builder.create();
				alert.show();
			}
		}
	}

	private void finishDialogNoBluetooth() {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage(R.string.alert_dialog_no_bt)
				.setIcon(android.R.drawable.ic_dialog_info)
				.setTitle(R.string.app_name)
				.setCancelable(false)
				.setPositiveButton(R.string.alert_dialog_ok,
						new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {
								finish();
							}
						});
		AlertDialog alert = builder.create();
		alert.show();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.option_menu, menu);
		return true;
	}

	private static final int REQUEST_CONNECT_DEVICE = 1;

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.connect: {
			Intent serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
		}
			return true;
		case R.id.preferences: {
			cancel();
			Bluemd.this.finish();
		}
			return true;
		}
		return false;
	}

	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_CONNECT_DEVICE:
			if (resultCode == RESULT_OK) {
				final String address = data
						.getStringExtra(DeviceListActivity.DEVICE_ADDRESS);
				BluetoothDevice device = mBluetoothAdapter
						.getRemoteDevice(address);
				mConnectedDeviceName = data
						.getStringExtra(DeviceListActivity.DEVICE_NAME);
				LogUtil.e("====>address = " + address + ", name = "
						+ mConnectedDeviceName);
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				final EditText input = new EditText(this);
				builder.setTitle("设备名称");
				builder.setMessage("请输入设备名称");
				input.setText(device.getName());
				builder.setView(input);
				if (preferences.getString(device.getAddress(), null) == null) {
					builder.setPositiveButton("Save",
							new DialogInterface.OnClickListener() {
								@Override
								public void onClick(DialogInterface dialog,
										int whichButton) {
									mConnectedDeviceName = input.getText()
											.toString();
									preferences
											.edit()
											.putString(address,
													mConnectedDeviceName)
											.commit();
									live(address);
									return;
								}
							});
					builder.create().show();
				} else {
					live(address);
				}
				alreadyUpdated = false;
			}
			break;
		case REQUEST_ENABLE_BT:
			if ((mBluetoothAdapter != null) && (!mBluetoothAdapter.isEnabled())) {
				finishDialogNoBluetooth();
			}
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		firstTimeThread.Stop();
		cancel();
	}

	private String mConnectedDeviceName = null;

	private void cancel() {
		if (firstTimeThread != null) {
			firstTimeThread.Stop();
		}
		if (mBluetoothLeService != null) {
			mBluetoothLeService.close();
			mBluetoothLeService = null;
		}
		unregisterReceiver(mGattUpdateReceiver);
		unbindService(mServiceConnection);
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage("Do you exit program?")
					.setIcon(android.R.drawable.ic_dialog_info)
					.setTitle(R.string.app_name)
					.setCancelable(false)
					.setPositiveButton("Yes",
							new DialogInterface.OnClickListener() {
								public void onClick(DialogInterface dialog,
										int id) {
									android.os.Process
											.killProcess(android.os.Process
													.myPid());
									Bluemd.this.finish();
								}
							})
					.setNegativeButton("No",
							new DialogInterface.OnClickListener() {

								@Override
								public void onClick(DialogInterface dialog,
										int which) {
								}

							});
			AlertDialog alert = builder.create();
			alert.show();
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	private void sendData(String hexValue) {
		if (!isConnected)
			return;
		if (mBluetoothLeService != null) {
			mBluetoothLeService.WriteString(hexValue);
		}
	}

	private void liveData(String hexValue) {
		if (!isConnected) {
			Intent serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
		} else {
			sendData(hexValue);
		}
	}

	private void live(String address) {
		if (isConnected) {
			mBluetoothLeService.disconnect();
			isConnected = false;
			mBluetoothLeService.connect(address);
		} else {
			mBluetoothLeService.connect(address);
		}
	}

}