package com.scpark.blecmd;

import java.util.ArrayList;

import android.R.integer;
import android.R.xml;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.drawable.shapes.Shape;
import android.util.AttributeSet;
import android.util.Log;
import android.util.Printer;
import android.view.MotionEvent;
import android.view.View;

public class RoundView extends View {

	private int mPointX = 0, mPointY = 0;
	private int angle = 0;
	private int progress;// (0-1)
	private double total = ANGLE_MAX - ANGLE_MIN;

	private Matrix panRotate = new Matrix();
	private Bitmap roundSelector;
	private Bitmap backgroundImage;

	private Point current = new Point();
	private Context context;

	private static final int ANGLE_MIN = 6;
	private static final int ANGLE_MAX = 252;
	private static final int PROGRESS_ALL = 255;

	private Point beginPoint = new Point();
	private RoundViewListener roundViewListener;
	private int beginAngle, beginPointAngle;

	ArrayList<Integer> filter = new ArrayList<Integer>();
	private boolean thisMovingFlag = true;
	private int lastAngle = 0;
	private boolean reachUnreadableFlag = false;
	private int reachUnreachableGrowingFlag = -1;
	private int growingFlag = 0; // -1 Null 0:--> 1:<--

	public RoundView(Context context, AttributeSet attrs) {
		super(context, attrs);
		// Log.i("WiFi","Text"+text);
		// width =
		// Integer.parseInt(attrs.getAttributeValue("http://schemas.android.com/apk/res/android",
		// "layout_width"));
		// height =
		// Integer.parseInt(attrs.getAttributeValue("http://schemas.android.com/apk/res/android",
		// "layout_height"));
		// mPointX = 160;
		// mPointY = 160;
		// Log.i("RoundView","Height:"+height+" Width:"+width);
		this.context = context;
		angle = 100;
		roundSelector = BitmapFactory.decodeStream(context.getResources()
				.openRawResource(R.drawable.test));
		backgroundImage = BitmapFactory.decodeStream(context.getResources()
				.openRawResource(R.drawable.back1));

	}

	public void setRoundViewListener(RoundViewListener roundViewListener) {
		this.roundViewListener = roundViewListener;
	}

	public RoundView(Context context, int px, int py, int radius) {
		super(context);
		// mPaint.setColor(Color.RED);
		// mPaint.setStrokeWidth(2);
		// setBackgroundResource(R.drawable.test);

		mPointX = px;
		mPointY = py;
		angle = radius;

		roundSelector = BitmapFactory.decodeStream(context.getResources()
				.openRawResource(R.drawable.test));

	}

	/*
	 * @Override public boolean dispatchTouchEvent(MotionEvent event) {
	 * Log.i("RoundView","Touching!"); invalidate(); return true; }
	 */

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		int action = event.getAction() & MotionEvent.ACTION_MASK;
		// Log.d("CV", "Action ["+action+"]");
		switch (action) {

		case MotionEvent.ACTION_DOWN: {
			// path.moveTo(event.getX(), event.getY());
			beginPoint.set((int) event.getX() - this.getWidth() / 2,
					(int) event.getY() - this.getHeight() / 2);

			beginPointAngle = (int) Tools.getAngle(beginPoint);
			beginAngle = angle;
			lastAngle = angle;
			Log.i("RoundView", "Angle: Begin Begin begin" + beginPointAngle
					+ " angle:" + angle);
			thisMovingFlag = true;
			growingFlag = 0;
			reachUnreadableFlag = false;
			filter.clear();
			// start.set((int)event.getX(),(int)event.getY());
			break;
		}
		case MotionEvent.ACTION_MOVE: {
			// path.lineTo(event.getX(), event.getY());
			/*
			 * Log.i("RoundView", "Action_Move! X:" + (event.getX() -
			 * this.getWidth() / 2) + " Y:" + (event.getY() - this.getHeight() /
			 * 2));
			 */

			if (thisMovingFlag) {
				current.set((int) event.getX() - this.getWidth() / 2,
						(int) event.getY() - this.getHeight() / 2);
				if (current.x == 0 || current.y == 0)
					return true;
				int angleTmp = (int) Tools.getAngle(current) - beginPointAngle
						+ beginAngle;
				/*
				 * if (angle > 35 && angle < 145) return true;
				 */

				angleTmp = angleTmp % 360;
				if (angleTmp < 0)
					angleTmp += 360;

				if (Math.abs(angle - ANGLE_MIN) < 10) {
					if (Math.abs(angleTmp - ANGLE_MAX) < 10)
						thisMovingFlag = false;
				}
				if (Math.abs(angle - ANGLE_MAX) < 10) {
					if (Math.abs(angleTmp - ANGLE_MIN) < 10)
						thisMovingFlag = false;
				}

				// Log.i("RoundView","Moving Flag:"+growingFlag);
				if (angleTmp < ANGLE_MIN) {
					reachUnreadableFlag = true;
					angleTmp = ANGLE_MIN;
				} else if (angleTmp > ANGLE_MAX) {
					reachUnreadableFlag = true;
					angleTmp = ANGLE_MAX;
				}

				if (Math.abs(lastAngle - angleTmp) > 200)
					return true;

				angle = angleTmp;
				progress = (int) ((angle - ANGLE_MIN) / total * PROGRESS_ALL);

				lastAngle = angle;
				Log.i("RoundView", "Angle: begin" + beginAngle + " angle:"
						+ angle + " progress:" + progress);

				// setProgress(progress);
				invalidate();
				if (roundViewListener != null) {
					roundViewListener.onMoved(progress);
				}
			}

			break;
		}

		}
		// invalidate();
		return true;
	}

	public void setProgress(double progress) {
		angle = (int) (progress * 1.0 / PROGRESS_ALL * total);
		Log.i("RoundView", "Angle in setProgress: " + angle);
		angle = angle + ANGLE_MIN;
		panRotate.setRotate(angle, roundSelector.getWidth() / 2,
				roundSelector.getHeight() / 2);
		invalidate();
	}

	@Override
	public void onDraw(Canvas canvas) {

		panRotate.setRotate(angle, roundSelector.getWidth() / 2,
				roundSelector.getHeight() / 2);
		canvas.drawBitmap(roundSelector, panRotate, null);

	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		// TODO Auto-generated method stub

		int parentWidth = MeasureSpec.getSize(widthMeasureSpec);
		int parentHeight = MeasureSpec.getSize(heightMeasureSpec);
		super.onMeasure(parentWidth, parentHeight);

		this.setMeasuredDimension(parentWidth, parentWidth);

		Log.v("onMeasure", "" + parentWidth + "   " + parentHeight);

		roundSelector = BitmapFactory.decodeStream(context.getResources()
				.openRawResource(R.drawable.test));
		backgroundImage = BitmapFactory.decodeStream(context.getResources()
				.openRawResource(R.drawable.back1));
		double rate = parentWidth * 1.0 / backgroundImage.getWidth();
		// rate = rate * 1.55;
		// rate = rate * 1.55;
		roundSelector = Bitmap.createScaledBitmap(roundSelector,
				(int) (rate * roundSelector.getWidth()),
				(int) (rate * roundSelector.getWidth()), true);
		// super.onMeasure(panpic.getWidth(), panpic.getHeight());
		Log.v("onMeasure", "" + this.getHeight() + "   " + this.getWidth());
		setMeasuredDimension(roundSelector.getWidth(),
				roundSelector.getHeight());
		// totalWidth=parentWidth;
	}

}
